<?php
/**
 * @category     Addons
 * @package      Admin tool: wbBackup
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.2
 * @lastmodified 2021/03/05
 * @since        File available since 2021/01/14
 * @description  display backend administration page
 *
 * class.Wbbackup.php
*/

namespace addon\wbBackup\Lib;

use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck,StopWatch};
use src\Security\{CsfrTokens, Randomizer, Password};
use Ifsnop\Mysqldump as IMysqldump;

use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\TwigFilter;

class Wbbackup
{

    private static $LineSep = "\n";
    private static $oReg;
    private static $oRequest;
    private static $oTrans;
    private static $oDb;
    private static $log_handle;
    protected static $aBackupSettings;
    protected static $aConfig = [];
    // -----------------------------------------------------------------------------
    /** prevent class from public instancing and get an object to hold extensions */
        private function  __construct() {
        }
    // -----------------------------------------------------------------------------
    /** prevent from cloning existing instance */
        private function __clone() {}

        ###############################################################################
        // read settings for this section (getBackupSettings)
        ###############################################################################
        /** read default setting for this admintool */
                static public function getBackupSettings(){
                    global $aDefaults;
                    self::$oReg = WBAdaptor::getInstance();
                    self::$oRequest = self::$oReg->getRequester();
                    self::$oDb = self::$oReg->getDatabase();
                    self::$oTrans = self::$oReg->getTranslate();
                    //$oDb = \database:::getInstance();
                        $aBackupSettings  = [];
                        $section_id = 0;
                        $sql  = 'SELECT `s_name`, `s_value` FROM `'.TABLE_PREFIX.'mod_wbbackup_settings` ';
//                        echo $sql;
                        if ($query = self::$oDb->query($sql)){
                            while ($row = $query->fetchRow(MYSQLI_ASSOC)) {
                                $aBackupSettings[$row['s_name']] = $row['s_value'];
                            }
                        }
                        // insert default_settings if no settings for this section avaiable
                        if (sizeof($aBackupSettings) == 0){
                            $aFieldsList = [
                                'section_id' => 0
                            ];
                            /* add default setting if not set for this section */
                            if (\is_array($aDefaults)) {
                                $aTmp = \array_diff_key($aDefaults, $aBackupSettings);
                                if ($aTmp){
                                    UpdateKeyValue('mod_wbbackup_settings', $aDefaults, '', $aFieldsList);
                                }
                            }
                        }

//            echo "<pre>";
//            print_r($aBackupSettings);
//            echo "</pre>";
                return $aBackupSettings;
                }
        // end of static public function ('getBackupSettings')
        // ----------------------------------------------------------------------------
        public static function getBackupSetting($sField):?string
        {
            $sRetval = '';
            $oDb = \database::getInstance();
            if (!empty($sField)){
                $sql= 'SELECT `s_value` FROM `'.$oDb->TablePrefix.'mod_wbbackup_settings` '
                    . 'WHERE `s_name` = \''.$oDb->escapeString($sField).'\' ';
                $sRetval = $oDb->get_one($sql);
                //
            }
            return ($oDb->get_error() ?: $sRetval);
        }

        public static function debugSettingInfo($mValue){
            print '<pre class="mod-pre"> filename: <span>'.basename(__FILE__).' -> '."\n";
            print_r( [gettype($mValue),$mValue] );
            print '</pre>'; \flush (); //  sleep(10); die();
        }

        ################################################################################
        // get status of dot files/folder usability, 1 or 0, default = 1 (getUseDotFiles)
        ###############################################################################
                public static function getUseDotFiles()
                {
                    $args = [
                        'use_dot'   => [
                              //'filter'   => (\FILTER_VALIDATE_BOOL ?? \FILTER_VALIDATE_BOOLEAN ),
                              'flags'    => \FILTER_NULL_ON_FAILURE,
                              'options'  => ['default' => false],
                              ],
                    ];
                    $val = self::getBackupSetting('use_dot');
                    //$val = ['use_dot' => self::getBackupSetting('use_dot')];
                    //$retVal = filter_var_array($val, $args);
                    // define const use by php 8.1
                    if (!defined('FILTER_VALIDATE_BOOL')) {define('FILTER_VALIDATE_BOOL' ,\FILTER_VALIDATE_BOOLEAN);}
                    $retVal = filter_var($val, FILTER_VALIDATE_BOOL, $args);
                    //self::debugSettingInfo($retVal);
                    return ($retVal ?? false);

                }
         // end of function_exists('getUseLogMode')

        ################################################################################
        // get status of log usability, 1 or 0, default = 1 (getUseLogMode)
        ###############################################################################
                public static function getUseLogMode()
                {
                    $val = self::getBackupSetting('use_log');
                    $retVal = filter_var($val, \FILTER_VALIDATE_BOOLEAN, ['flags' => \FILTER_NULL_ON_FAILURE]);
                    //self::debugSettingInfo($retVal);
                    return ($retVal ?? false);
                }
         // end of function_exists('getUseLogMode')

        ################################################################################
        // get status of log usability, 1 or 0, default = 1 (getLogExludedDirs)
        ###############################################################################
                public static function getLogExludedDirs()
                {
                    $val = self::getBackupSetting('log_excluded_dirs');
                    $retVal = filter_var($val, \FILTER_VALIDATE_BOOLEAN, ['flags' => \FILTER_NULL_ON_FAILURE]);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getLogExludedDirs')
         //

        ################################################################################
        // get status of log usability, 1 or 0, default = 1 (getLogExludedLargeFiles)
        ###############################################################################
                public static function getLogExludedLargeFiles()
                {
                    $val = self::getBackupSetting('log_excluded_large_files');
                    $retVal = filter_var($val, \FILTER_VALIDATE_BOOLEAN, ['flags' => \FILTER_NULL_ON_FAILURE]);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getLogExludedLargeFiles')
         //

        ################################################################################
        // get excluded folders, default = '' (getExCludedDirs)
        ###############################################################################
                public static function getExCludedDirs()
                {
                    $val = self::getBackupSetting('excluded_dirs');
                    $retVal = filter_var($val, \FILTER_SANITIZE_FULL_SPECIAL_CHARS, \FILTER_NULL_ON_FAILURE);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getExCludedDirs')
         //

        ################################################################################
        // get file extension to skipped from backup, 1 or 0, default = '' (getExCludedExtensions)
        ###############################################################################
                public static function getExCludedExtensions()
                {
                    $val = self::getBackupSetting('excluded_extension');
                    $retVal = filter_var($val, \FILTER_SANITIZE_FULL_SPECIAL_CHARS, \FILTER_NULL_ON_FAILURE);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getExCludedExtensions')
         //

        ################################################################################
        // get size for excluded files, default = 20, means 20 Mb (getMaxFileSize)
        ###############################################################################
                public static function getMaxFileSize(){
                    $val = self::getBackupSetting('max_file_size');
                    $retVal = filter_var($val, \FILTER_SANITIZE_NUMBER_FLOAT , \FILTER_FLAG_ALLOW_FRACTION );
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }

        ################################################################################
        // get backup folder (getBackupFolder)
        ###############################################################################
                public static function getBackupFolder()
                {
                    $val = self::getBackupSetting('backup_dir');
                    $retVal = filter_var($val, \FILTER_SANITIZE_FULL_SPECIAL_CHARS, \FILTER_NULL_ON_FAILURE);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getBackupFolder')
         //

        ###############################################################################
        // get Use_DropTable in mysql backup file, default = 1 (getUseDropTableMode)
        ###############################################################################
                public static function getUseCompressMode()
                {
                    $val = self::getBackupSetting('use_compress');
                    $retVal = filter_var($val, \FILTER_SANITIZE_FULL_SPECIAL_CHARS, \FILTER_NULL_ON_FAILURE);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getUseCompressMode')
         //

        ###############################################################################
        // get Use_DropTable in mysql backup file, default = 1 (getUseDropTableMode)
        ###############################################################################
                public static function getUseCharacterSet()
                {
                    $val = self::getBackupSetting('use_character_set');
                    $retVal = filter_var($val, \FILTER_SANITIZE_FULL_SPECIAL_CHARS, \FILTER_NULL_ON_FAILURE);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getUseCharacterSet')

        ###############################################################################
        // get Use_DropTable in mysql backup file, default = 1 (getUseDropTableMode)
        ###############################################################################
                public static function getUseDropTableMode()
                {
                    $val = self::getBackupSetting('use_droptable');
                    $retVal = filter_var($val, \FILTER_VALIDATE_BOOLEAN, ['flags' => \FILTER_NULL_ON_FAILURE]);
                    //self::debugSettingInfo($retVal);
                    return $retVal;
                }
         // end of function_exists('getUseDropTableMode')
         //

        #################################################################################
        // change filzesize #(bytes) into a readable format (human_filesize)
        ###############################################################################

                public static function human_filesize($size, $precision = 2)
                {
                    $units = ['Bytes','kB','MB','GB','TB','PB','EB','ZB','YB'];
                    $step = 1024;
                    $i = 0;
                    while (($size / $step) > 0.9) {
                        $size = $size / $step;
                        $i++;
                    }
                    return round($size, $precision).' '.$units[$i];
                }

        public static function getServerVars($sVarName)
        {
            return (self::$oRequest->getServerVar(\strtoupper($sVarName)));
        }

        public static function initBackupVars()
        {
            //
                    $iMaxFileSize           = self::getMaxFileSize();
                    $sBackupFolder          = self::getBackupFolder();
                    $sBackupDirPath         = self::$oReg->AppPath.$sBackupFolder.'/';
                    $sExcludedDirectories   = self::getExCludedDirs();
                    $sLogExludedDirs        = self::getLogExludedDirs();
                    $sLogExludedLargeFiles  = self::getLogExludedLargeFiles();
                    $sExcludedExtensions    = self::getExCludedExtensions();
                    $host = self::getServerVars('SERVER_NAME'); // SERVER_NAME
                    $timestr   = PreCheck::dateFormatToStrftime("Ymd_His", time());
                    //$timestr = \PHP81_BC\strftime("%Y%m%d_%H%M%S", time());

                    StopWatch::start();
                    self::$aConfig['starttime'] = StopWatch::startTime();
                    $sStartTimeDate = date('[Y-m-d H:i:s e]', (int)self::$aConfig['starttime']);

                    //self::$aConfig['sStartTimeDate'] = $sStartTimeDate;
                    //self::$aConfig['zip_file'] = basename(self::$aConfig['zip_path']);

                    return self::$aConfig  = [
                        'zip_path'  => $sBackupDirPath.$host.'-'.$timestr.'.zip',
                        'file_path' => $sBackupDirPath.$host.'-'.$timestr.'.zip',
                        'zip_file'  => $host.'-'.$timestr.'.zip',
                        'FileSize'  => 0,
                        'sStartTimeDate' => $sStartTimeDate,
                        'sEndTimeDate' => '',
                        'folders' => 0,
                        'files' => 0,
                        'starttime' => time(),
                        'StartDate' => '',
                        'endtime' => 0,
                        'EndDate' => '',
                        'ExecutionTime' => 0,
                        'iMaxFileSize' => $iMaxFileSize,
                        'sBackupFolder' => $sBackupFolder,
                        'sLogFile' => '',
                        'aExcludedDirectories' => PreCheck::convertToArray($sExcludedDirectories),
                        'sLogExludedDirs' => $sLogExludedDirs,
                        'sLogExludedLargeFiles' => $sLogExludedLargeFiles,
                        'aExcludedExtensions' => PreCheck::convertToArray($sExcludedExtensions),
                        'sDownloadUrl' => self::$oReg->AppUrl.''.$sBackupFolder.'/'.$host.'-'.$timestr.'.zip',
                        'host' => $host,
                        'timestr' => $timestr,
                        'sFilename' => $host.'-'.$timestr.'',
                    ];
        }

        ################################################################################
        // build mysql backup as zip or sql-file (buildSqlBackup)
        // based on https://github.com/ifsnop/mysqldump-php
        ################################################################################
                public static function buildSqlBackup($sBackupContent)
                {
                    $aParameters = self::initBackupVars();

                    $sBackupFolder     = self::getBackupFolder();
                    make_dir(self::$oReg->AppPath.$sBackupFolder);

                    $sBackupDirPath    = self::$oReg->AppPath.$sBackupFolder.'/';
                    //Enter your database information here and the name of the backup file
                    $mysqlDatabaseName = self::$oReg->DbName;
                    $mysqlUserName     = self::$oReg->DbUsername;
                    $mysqlPassword     = self::$oReg->DbPassword;
                    $mysqlHostName     = self::$oReg->DbHost;
                    $sDropTableMode    = self::getUseDropTableMode();
                    $bCompress         = self::getUseCompressMode();
                    $sSQLBackupFilname = self::$aConfig['host'].'-dbbackup-'.self::$aConfig['timestr'].(($bCompress === 'Gzip') ? '.sql.gz' : '.sql');
                    $mysqlExportPath   = self::$oReg->AppPath.''.$sBackupFolder.'/'.$sSQLBackupFilname;
                    $mysqlCompress     = (($bCompress==='None') ? IMysqldump\Mysqldump::NONE : IMysqldump\Mysqldump::GZIP);
                    $mysqlCaracterSet  = ((self::getUseCharacterSet()==='utf8') ? IMysqldump\Mysqldump::UTF8 : IMysqldump\Mysqldump::UTF8MB4 );
                    $aInludeTables = [];
                    if (isset($sBackupContent) && ($sBackupContent == 'DBallTables')){/* do nothing */;}
                    if (isset($sBackupContent) && ($sBackupContent == 'DBwbonly')){
                        //self::$oReg->TablePrefix.$mysqlDatabaseName;
                            $sPattern = addcslashes ( self::$oDb->TablePrefix, '%_' );
                            $sql = 'SHOW TABLES LIKE "'.$sPattern.'%"';
                            $result = self::$oDb->query($sql);
                            // Loop through tables
                            $i = 0;
                            if (($aTables = $result->fetchAll()))
                            {
                               for ($i=0; $i < count($aTables); $i++) {
                                  if (!is_null($aTables[$i])){
                                      $aInludeTables = array_merge($aInludeTables,array_values($aTables[$i]));
                                  }
                               }//end for
                            }
                    }
                    try {
                        $aOption = [
                            'add-drop-table' => $sDropTableMode,
                            'if-not-exists'  => (($sDropTableMode==false) ? true : false ),
                            'compress' => $mysqlCompress,
                            'default-character-set' => $mysqlCaracterSet,
                            'include-tables' => $aInludeTables,
                            'complete-insert' => true,
                        ];
                        $dump = new IMysqldump\Mysqldump('mysql:host='.$mysqlHostName.';dbname='.$mysqlDatabaseName.'', $mysqlUserName, $mysqlPassword,$aOption);
                        $dump->start($mysqlExportPath);
                        //
                    } catch (\Exception $e) {
                        echo 'mysqldump-php error: ' . $e->getMessage();
                    }
                }

        ################################################################################
        // scan Webspace files/folder
        ################################################################################
        public static function loadBackupTheme($aTplData)
        {
                $sAddonThemePath = str_replace('\\','/',dirname(__DIR__)).'/themes/default/';
                // create twig template object
                $loader = new \Twig\Loader\FilesystemLoader($sAddonThemePath);
                $twig   = new \Twig\Environment($loader, [
                    'autoescape'       => false,
                    'cache'            => false,
                    'strict_variables' => false,
                    'debug'            => false,
                    'auto_reload'      => true,
                ]);
                return $twig->render('backup.twig', $aTplData);
        }

        protected static function scanIterator ($sStartPath)
        {
            $Directory  = new \RecursiveDirectoryIterator(
                              $sStartPath,
                              \FilesystemIterator::CURRENT_AS_SELF |
                              \FilesystemIterator::FOLLOW_SYMLINKS |
                              \FilesystemIterator::KEY_AS_PATHNAME |
                              \FilesystemIterator::UNIX_PATHS |
                              (self::getUseDotFiles() ? 0 : \FilesystemIterator::SKIP_DOTS)
                          );
            $oIterator   = new \RecursiveIteratorIterator(
                              $Directory,
                              \RecursiveIteratorIterator::SELF_FIRST,
                              \RecursiveIteratorIterator::CATCH_GET_CHILD
                          );
          return $oIterator;
        }
/* --------------------------------------------------------------------------- */

        public static function writeLogFile($sError='')
        {
            if (!empty($sError) && self::$aConfig['sLogExludedLargeFiles'] == 'true') {
                fwrite(self::$log_handle, $sError);
            }
        }

        public static function getBackupVars()
        {

              return self::$aConfig;
        }
        ################################################################################
        // build webspace backup as zip (buildZipBackup)
        ################################################################################

                static public function buildZipBackup($aExCludedDirs) {

                    // Make directory in the root of this wB for the backups if not yet existing
                    $sBackupDir = self::$aConfig['sBackupFolder'];
                    $sBackupDirPath = self::$oReg->AppPath.''.$sBackupDir;
                    //if (!\function_exists('make_dir')){require(self::$oReg->AppPath.'framework/functions.php');}
                    make_dir($sBackupDirPath);
                    // Backup log
                    $sLogFile = $sBackupDirPath.'/'.self::$aConfig['host'].'-'.self::$aConfig['timestr'].'.log';
                    //echo nl2br (sprintf("%d %s \n",__LINE__,$sLogFile));
                    if (!(self::$log_handle = fopen($sLogFile, 'w')))
                    {
                        echo nl2br (sprintf("%d %s \n",__LINE__,'Error open log file'));
                    }

                    $iCount = 0;

                    $sStartTimeDate = date('[Y-m-d H:i:s e]', self::$aConfig['starttime']);
                    self::$aConfig['sStartTimeDate'] = $sStartTimeDate;
                    self::$aConfig['zip_file'] = basename(self::$aConfig['zip_path']);
                    $bExcludeDotFiles = self::getUseDotFiles();
                    fwrite(self::$log_handle, $sStartTimeDate.' Backup started'.PHP_EOL);
                    // start the backup at the root of WB
                    $sStartPath                 = self::$oReg->AppPath;
                    // Specify directories you do not want in the backup as an array
                    $aExcludedDirectories       = self::$aConfig['aExcludedDirectories'];
                    // Specify file extensions you do not want in the backup as an array
                    $aExcludedExtensions        = self::$aConfig['aExcludedExtensions'];
                    // Log excluded directories in the log file
                    $log_excluded_dirs          = self::$aConfig['sLogExludedDirs'];
                    $log_excluded_large_files   = self::$aConfig['sLogExludedLargeFiles'];
                    // ignored file extensions
//                    $ignore_exts                = array();
                    // Instantiate native php class ZipArchive
                    $aData = [
                        'oReg' => self::$oReg,
                        'aConfig' => self::$aConfig,
                    ];

                    $zip = new \ZipArchive();
                    if ($zip->open(self::$aConfig['zip_path'], \ZIPARCHIVE::CREATE) === TRUE)
                    {
                        fwrite(self::$log_handle, date('[Y-m-d H:i:s e]').' Class ZipArchive: Opened zip'.PHP_EOL);
                        chdir($sStartPath);
                        $oIterator = self::scanIterator($sStartPath);

                        foreach ($oIterator as $file) {
                            $filename  = $file->getFilename();
                            $filepath  = $file->getPathname();
                            $rootpath  = \str_replace(self::$oReg->AppPath, '', $file->getPathname());

                            // Skip excluded directories -----------------------------
                            $bLoop = FALSE;
                            foreach ($aExcludedDirectories as $exclude)
                            {
                                if (strpos($filepath, $exclude) !== FALSE || ($rootpath === $exclude))
                                {
                                    $bLoop = TRUE;
                                    //echo \nl2br(\sprintf("---- [%04d] %s \n",__LINE__,$filepath));
                                    if ($log_excluded_dirs == TRUE)
                                    {
                                        $sMsg = sprintf("%s Skipped [%s]: Directory in ignore list\n",date('[Y-m-d H:i:s e]'), $rootpath);
                                        fwrite(self::$log_handle, $sMsg);
                                    }
                                    break;
                                }
                            }
                            if ($bLoop){continue;}

                            // Skip not allowed extensions ---------------------------
                            //$sCurrentExt = strtolower(pathinfo($filepath, PATHINFO_EXTENSION));
                            $sCurrentExt = rtrim(PreCheck::getExtension($filepath),'.');
                            if ((!empty($sCurrentExt)) && in_array($sCurrentExt,$aExcludedExtensions)) {
                                    //echo \nl2br(\sprintf("---- [%04d] %s \n",__LINE__,$filepath));
                                if ($log_excluded_dirs == 'true') {
                                    $sMsg = sprintf("%s Skipped [%s]: File Extension in ignore list\n",date('[Y-m-d H:i:s e]'), $rootpath);
                                    fwrite(self::$log_handle, $sMsg);
                                }
                                continue;
                            }

                            if (($bExcludeDotFiles == FALSE) && (strpos($filepath, '/.') !== FALSE)) {
                                //echo nl2br(sprintf("%s\n",$filepath));
                                continue;
                            }

                            // Skip double-dots that refer to parent directory -------
                            if (($filename === '..') || ($filename === '.')) {continue;}

                            // Skip large files -------------------------------------
                            if (is_file($filepath))
                            {
                                self::$aConfig['FileSize'] = (int)filesize($filepath);
                                $max_size_mb = (int)self::$aConfig['iMaxFileSize'] *1024 * 1024; // MB
                                if ((int)self::$aConfig['FileSize'] > (int)$max_size_mb) {
                                    if (self::$aConfig['sLogExludedLargeFiles'] == 'true') {
                                        $sMsg = sprintf("%s Skipped [%s]: Exceeded max file size\n",date('[Y-m-d H:i:s e]'), $rootpath);
                                        fwrite(self::$log_handle, $sMsg);
                                    }
                                    continue;
                                }
                            }

                            // Add empty directories to the zip ----------------------
                            if ($file->isDir()) {
                                $aTime  = $file->getATime();
                                $iMTime = $file->getMTime();
                                $modifyTimestamp = $iMTime;
                                $accessTimestamp = (($aTime === null) ? null : $aTime);
                                $rootpath .= '/';
                                if ($zip->addEmptyDir($rootpath))
                                {
                                    //echo nl2br(sprintf("[%05d] <b>addEmptyDir('%s')</b> \n",__LINE__,$rootpath));
                                    ++self::$aConfig['folders'];
                                }
                                if (\PHP_MAJOR_VERSION === 8 && $zip->setMtimeName($rootpath,$modifyTimestamp) === true) //$iMTime
                                {
                                    //echo nl2br(sprintf("[%05d] <b>setMtimeName('%s',%s)</b> \n",__LINE__,$rootpath, date('Y-m-d H:i',$modifyTimestamp)));
                                }
                                else
                                {
                                    //echo nl2br(sprintf("[%05d] <b>setMtimeName('%s',%s)</b> FAILED \n",__LINE__,$rootpath, date('Y-m-d H:i',$modifyTimestamp)));
                                }
                            }
                            else  // Add the files to the zip
                            {
                                chdir(self::$oReg->AppPath);
                                if ($zip->addFile($rootpath, $rootpath)){
                                    //echo nl2br(sprintf("[%05d] addFile('%s', '%s') \n",++$iCount,$rootpath,$rootpath));
                                    ++self::$aConfig['files'];
                                }
                            }
                            // Add the files from the root dir
                        }//foreach $oIterator / $aRootFolderList

                        // Get error
// ------------------------------ */
                        $status = $zip->getStatusString();
                        if ($zip->close() === false) {
                            \fwrite(self::$log_handle, date('[Y-m-d H:i:s e]').' Failed to create file backup. Error creating zip archive: '.$status.PHP_EOL);
                            die(\json_encode(array('code' => 403, 'message' => 'Failed to create file backup')));
                        }
                    }// end zip->open
                    PreCheck::resetMaxExecutionTime();
                    \flush();\clearstatcache();

                    self::$aConfig['endtime'] = StopWatch::endTime();
                    $fExecutiontime = StopWatch::stop();
                    //echo \nl2br(\sprintf("---- [%04d] %s \n",__LINE__,$fExecutiontime));
                    self::$aConfig['executionTime'] = $fExecutiontime;
                    $sEndTimeDate = date('[Y-m-d H:i:s e]', (int)self::$aConfig['endtime']);
                    \fwrite(self::$log_handle, \sprintf('%s Backup finished successfully, added %d Folder(s) and %d File(s) to %s', $sEndTimeDate,self::$aConfig['folders'],self::$aConfig['files'],\basename(self::$aConfig['zip_path'])).PHP_EOL);
                    \fwrite(self::$log_handle, \sprintf('Backup Execution time %s seconds',self::$aConfig['executionTime']).PHP_EOL);
                    \fclose(self::$log_handle);
                    $iList = 0;
                    self::$aConfig['FileSize'] = (\is_readable(self::$aConfig['zip_path']) ? \filesize(self::$aConfig['zip_path']) : 0);
                    $aOutput = [
                    'Archive'       => self::$aConfig['zip_path'],
                    'FileSize'      => self::$aConfig['FileSize'],
                    'StartTime'     => self::$aConfig['starttime'],
                    'EndTime'       => self::$aConfig['endtime'],
                    'StartDate'     => $sStartTimeDate,
                    'EndDate'       => $sEndTimeDate,
                    'folders'        => self::$aConfig['folders'],
                    'files'         => self::$aConfig['files'],
                    'ExecutionTime' => self::$aConfig['executionTime'].''
                    ];
                    self::$aConfig = \array_merge(self::$aConfig,$aOutput);

                }// end of function buildZipBackup


        ################################################################################
        // get size for excluded files, default = 20, means 20 Mb (getMaxFileSize)
        ###############################################################################

                static public function getAllBackupFiles($sBackupFolder){
                    global $TEXT, $MOD_BACKUP, $admin;
                    $aResult                    = [];
                    $sBackupDirPath             = self::$oReg->AppPath.''.$sBackupFolder;
                    $sBackupDirUrl              = self::$oReg->AppUrl.''.$sBackupFolder;
                    if (\is_dir($sBackupDirPath)) {
                        $sBackupZipFiles = scan_current_dir($sBackupDirPath, '/.*\.zip/');
                        $sBackupSqlFiles = scan_current_dir($sBackupDirPath, '/.*\.sql\.gz|.*\.sql/i');
                    }

                    $iZipCounter = 0;
                    $iSqlCounter = 0;
                    // zip file list
                        if (!empty($sBackupZipFiles['filename']) &&(\count($sBackupZipFiles['filename']) > 0)) //
                        {
                            $matches = [];
                            foreach ($sBackupZipFiles['filename'] as $sZipFile) {
                                $sCurrentZipFile = [];
                                if (!\is_writable($sBackupDirPath.'/'.$sZipFile)){continue;}
                                \preg_match('#(\d{8,})_(\d{6,})#', $sZipFile, $matches);
                                if (empty($matches)){unset($oDate);continue;}
                                $oDate = new \DateTime($matches[1].' '.$matches[2]);
                                $oDate->setTimezone(new \DateTimeZone('Europe/Berlin'));
                                $date  = $oDate->format('Y-m-d H:i:s');
                                $aCurrentZipFile['filelink']          = $sBackupDirUrl.'/'.$sZipFile;
                                $aCurrentZipFile['filename']          = $sZipFile;
                                $aCurrentZipFile['filenameidkey']     = $admin->getIDKEY($sZipFile);
                                $aCurrentZipFile['filedate']          = $date;
                                //$sCurrentZipFile['filedate']          = gmdate($sDefaultDateFormat.', '.$sDefaultTimeFormat, strtotime($date)+ $sDefaultTimeZone);//
                                $iCurrentZipFilesize                  = \filesize($sBackupDirPath.'/'.$sZipFile);
                                $aCurrentZipFile['filesize']          = self::human_filesize($iCurrentZipFilesize);
                                $sLogFileName                         = \str_replace('.zip', '.log', $sZipFile);
                                $aCurrentZipFile['logfilelink']       = $sBackupDirUrl.'/'.$sLogFileName;
                                $aCurrentBackupFiles[] = $aCurrentZipFile;
                                $iZipCounter++;
                            }
                            unset($oDate);
                            $aResult['ZIPlist']                       = ($aCurrentBackupFiles ?? []);
                        }
                    // sql file list -------------------------------------------------

                        if (!empty($sBackupSqlFiles['filename']) && (\count($sBackupSqlFiles['filename']) > 0)) {
                            $matches = [];
                            foreach ($sBackupSqlFiles['filename'] as $sSqlFile) {
                                $iSqlCounter++;
                                $aCurrentSqlFile  = [];
                                //preg_match('#(\d{10,})#', $sSqlFile, $match);
                                \preg_match('#(\d{8,})_(\d{6,})#', $sSqlFile, $matches);
                                if (empty($matches)){unset($oDate);continue;}
                                $oDate = new \DateTime($matches[1].' '.$matches[2]);
                                $oDate->setTimezone(new \DateTimeZone('Europe/Berlin'));
                                $date  = $oDate->format('Y-m-d H:i:s');

                                $aCurrentSqlFile['filelink']      = $sBackupDirUrl.'/'.$sSqlFile;
                                $aCurrentSqlFile['filename']      = $sSqlFile;
                                $aCurrentSqlFile['filenameidkey'] = $admin->getIDKEY($sSqlFile);
                                $aCurrentSqlFile['filedate']      = $date;
                                //$sCurrentSqlFile['filedate']    = gmdate($sDefaultDateFormat.', '.$sDefaultTimeFormat, $match[1] + $sDefaultTimeZone);
                                $iCurrentSqlFilesize              = \filesize($sBackupDirPath.'/'.$sSqlFile);
                                $aCurrentSqlFile['filesize']      = self::human_filesize($iCurrentSqlFilesize,3);
                                $aCurrentBackupSqlFiles[] = $aCurrentSqlFile;
                            }
                            unset($oDate);
                            $aResult['SQLlist'] = $aCurrentBackupSqlFiles;
                        }
                        $aResult['zipcounter']  = $iZipCounter;
                        $aResult['sqlcounter']  = $iSqlCounter;
                        return $aResult;
                    }

} //end of class

// end of file
