<?php
/**
 *
/**
 * @category     Addons
 * @package      Admin tool: wbBackup
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.2
 * @lastmodified 2021/03/05
 * @since        File available since 2021/01/14
 * @description  display backend administration page
 * Tool.inc.php
 */

use addon\wbBackup\Lib\Wbbackup;
use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck};

use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\TwigFilter;

//
//            echo "<pre>POST in tool.inc.php:<br>";
//            print_r($_POST);
//            echo "</pre>";
/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
/* -------------------------------------------------------- */
    flush();
//load some initial datas
    if (is_readable($sAddonPath.'/init.php')) {require ($sAddonPath.'/init.php');}
// if autoload failed


    $oTrans->enableAddon('modules\\\\'.$sAddonName);
    require(dirname(dirname(__FILE__))."/info.php");

    $sModuleVersion = $module_version;

    $js_back = ADMIN_URL .'/admintools/tool.php?tool="wbBackup';
    if( !$admin->get_permission($sAddonName,'module' ) ) {
             $admin->print_error($MESSAGE['ADMIN_INSUFFICIENT_PRIVELLIGES'], $js_back);
    }
    $sWbVersion = (defined('VERSION') ? VERSION : WB_VERSION);

// secure check - correct FTAN?
    if (!bin\SecureTokens::checkFTAN ()) {
        $aMessage = \sprintf("%s\n","Backup.inc.php - Missing FTAN ".$MESSAGE['GENERIC_SECURITY_ACCESS']);
        $admin->print_error($aMessage, ($sAddonBackUrl ?? $js_back));
    }

    $sTanCheck = $admin->checkIDKEY('sendTAN', false,'POST');
    if($sTanCheck === false ) {
        $aMessage = \sprintf("%s\n","Missing SendTan in Backup.inc.php - Missing IDKey ".$MESSAGE['GENERIC_SECURITY_ACCESS']);
        $admin->print_error($aMessage, $js_back );
    }

    PreCheck::increaseMemory('1024M','800');

    if (isset($sTanCheck) && ($sTanCheck == 'DBwbonly')) {
        $sBackupContent   = 'DBwbonly';
        $sSuccessText    = $oTrans->MOD_BACKUP_ADD_SQL_BACKUP;
    }
    if (isset($sTanCheck) && ($sTanCheck == 'DBallTables')) {
        $sBackupContent   = 'DBallTables';
        $sSuccessText     = $oTrans->MOD_BACKUP_ADD_SQL_BACKUP;
    }
    if (isset($sTanCheck) && ($sTanCheck == 'FileBackup')) {
        $sBackupContent   = 'FileBackup';
        $sSuccessText     = $oTrans->MOD_BACKUP_ADD_ZIP_BACKUP;
    }
    PreCheck::resetMaxExecutionTime();

    $FTAN = $admin->getFTAN('');
    $redirect_timer = ((\defined( 'REDIRECT_TIMER' ) && (REDIRECT_TIMER <= 10000)) ? REDIRECT_TIMER : false);
    // complete array with replacements
    $aTplDefaults = [
            'WB_URL'             => WB_URL,
            'ADMIN_URL'          => ADMIN_URL,
            'FTAN_NAME'          => $FTAN['name'],
            'FTAN_VALUE'         => $FTAN['value'],
            'MODULE_THEME_URL'   => $sAddonThemeUrl,
            'MODULE_THEME_PATH'  => $sAddonThemePath,
            'ICON_URL'           => $sAddonThemeUrl.'/images',
            'ADDON_NAME'         => strtolower($sAddonName),
            'ADDONNAME'          => strtoupper($sAddonName),
            'ADDONURL'           => $sAddonUrl,
            'LANGUAGE'           => LANGUAGE,
            'ICON_URL'           => $sAddonThemeUrl.'/images',
            'THEME_URL'          => $sAddonThemeUrl,
            'BACKUP_URL'         => $sAddonUrl.'/cmd/backup-sql.inc.php',
            'SAVE_URL'           => ADMIN_URL.'/pages/save.php',
            'TOOL_URL'           => ADMIN_URL.'/admintools/tool.php?tool=wbBackup',
            'CONFIRM'            => $oTrans->GENERIC_PLEASE_BE_PATIENT,
            'BACKUPFILE'         => $admin->getIDKEY('FileBackup'),
            'BACKUPALLDB'        => $admin->getIDKEY('DBallTables'),
            'BACKUPWBDB'         => $admin->getIDKEY('DBwbonly'),
            'REDIRECT_TIMER'     => $redirect_timer,
            'REDIRECT_URL'       => ADMIN_URL.'/admintools/tool.php?tool=wbBackup',
            'SUCCESSFULL_SAVED'  => $sSuccessText,
            'TEXT_CLOSE'         => $oTrans->MOD_BACKUP_TEXT_CLOSE ,
            'DbName'             => DB_NAME,
    ];

    $sDefaultDateFormat      = DEFAULT_DATE_FORMAT;
    $sDefaultTimeFormat      = DEFAULT_TIME_FORMAT;
    $sDefaultTimeZone        = DEFAULT_TIMEZONE;
    $sTablePrefix            = TABLE_PREFIX;
    $aTwigdata = $aTplDefaults;

//    $aTwigdata['DelFiletype']  = $aFiletype;
    $aTwigdata['setting'] = Wbbackup::getBackupSettings();
    $aData['aConfig'] = Wbbackup::initBackupVars();

/* */
    $sBackupFolder           = $aData['aConfig']['sBackupFolder'];
    $iMaxFileSize            = $aData['aConfig']['iMaxFileSize'];
    $sExCludedDirs           = $aData['aConfig']['aExcludedDirectories'];
    //$sDropTableMode          = $aData['aConfig']['sDropTableMode'];

    $aTwigdata['doAction']  = 'doBackup';
    // build the file list
//    $aTwigdata['Filelist']    = Wbbackup::getAllBackupFiles($sBackupFolder,$sDefaultDateFormat,$sDefaultTimeFormat,$sDefaultTimeZone, $sTablePrefix);
// database-backup - write a sql-file
    if (isset($sTanCheck) && (($sTanCheck == 'DBwbonly') || ($sTanCheck == 'DBallTables'))) {
        Wbbackup::buildSqlBackup($sBackupContent);
        //Wbbackup::buildSqlBackup($sBackupFolder,$sBackupContent,$sDefaultDateFormat,$sDefaultTimeFormat,$sDefaultTimeZone, $sTablePrefix, $sModuleVersion);
    }
    else
// filesystem-backup - write a zip-file
    if (isset($sTanCheck) && ($sTanCheck == 'FileBackup')) {
        $aData['aConfig'] = Wbbackup::initBackupVars();
        //Wbbackup::getBackupVars();

        echo Wbbackup::loadBackupTheme($aData);flush();
        clearstatcache();
        Wbbackup::buildZipBackup($sBackupFolder);//,$iMaxFileSize,$aExCludedDirs
        //echo Wbbackup::loadBackupTheme($aData);flush();
        $aData['aConfig'] = Wbbackup::getBackupVars();

        $iList = 0;
        echo \nl2br(\sprintf("%04d.) ArchiveFile %s \n",++$iList,$aData['aConfig']['zip_file']));
        echo \nl2br(\sprintf("%04d.) FileSize %s \n",++$iList,Wbbackup::human_filesize((int)$aData['aConfig']['FileSize'])));
        echo \nl2br(\sprintf("%04d.) Backup started at %s \n",++$iList,$aData['aConfig']['StartDate']));
        echo \nl2br(\sprintf("%04d.) Backup ends at %s \n",++$iList,$aData['aConfig']['EndDate']));
        echo \nl2br(\sprintf("%04d.) Added %s Folder(s) \n",++$iList,$aData['aConfig']['folders']));
        echo \nl2br(\sprintf("%04d.) Added %s File(s) \n",++$iList,$aData['aConfig']['files']));
        echo \nl2br(\sprintf("%04d.) Backup Execution time %s seconds \n",++$iList,$aData['aConfig']['ExecutionTime']));
        flush();clearstatcache();

    }
//die('Stop ZipArchive');
    if (version_compare($sWbVersion, '2.13.0', '>=')){
        if (/*class_exists('\Twig\Environment') &&*/ is_readable($sAddonThemePath.'/success.twig'))
            {
              //  create twig template object
                $loader = new \Twig\Loader\FilesystemLoader($sAddonThemePath);
                $oTwig  = new \Twig\Environment($loader, [
                    'autoescape'       => false,
                    'cache'            => false,
                    'strict_variables' => false,
                    'debug'            => false,
                    'auto_reload'      => true,
                ]);
            }
            $oTwig->addGlobal('lang', array_merge($TEXT, $MOD_BACKUP));
            //$oTwig->addFilter(new \Twig\TwigFilter('strftime', 'strftime_filter'));
            $oTwig->addFilter(new \Twig\TwigFilter('var_dump','var_dump'));
            echo $oTwig->render('success.twig', $aTwigdata);
        } else{
            $oTwigLoader = new Twig_Loader_Filesystem($sAddonThemePath);
            $oTwig = new Twig_Environment($oTwigLoader, array(
                'autoescape'       => false,
                'cache'            => false,
                'strict_variables' => false,
                'debug'            => true
            ));
            $oTwig->addGlobal('lang', array_merge($TEXT, $MOD_BACKUP));
            $oTpl = $oTwig->loadTemplate('success.twig');
            $oTpl->display($aTwigdata);
        }

// end of file
