<?php
/**
 * @category     Addons
 * @package      Admin tool: wbBackup
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.2
 * @lastmodified 2021/03/05
 * @since        File available since 2021/01/14
 * @description  display backend administration page
 *
/**
 * Install.inc.php
/* -------------------------------------------------------- */
declare(strict_types=1);

use bin\{WbAdaptor,SecureTokens,Sanitize,Requester};
use bin\helpers\{PreCheck,msgQueue};
use src\Security\{CsfrTokens,Randomizer};
use bin\Requester\HttpRequester;

// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
/* -------------------------------------------------------- */
if (!\is_callable('mod_wbBackup_install')){
    function mod_wbBackup_install($bDebug=false) {
        //global $OK ,$FAIL; // from core upgrade-script

        $sAddonFile   = \str_replace('\\','/',__FILE__);
        $sFolder      = \basename(\dirname($sAddonFile));
        switch ($sFolder):
            case 'cmd':
              $sAddonPath   = \dirname($sAddonFile,2).'/';
              break;
            default :
              $sAddonPath   = (\dirname($sAddonFile)).'/';
        endswitch;

        $sModulesPath = \dirname($sAddonPath).'/';
        $sModuleName  = \basename($sModulesPath);
        $sAddonName   = \basename($sAddonPath);
        $ModuleRel    = ''.$sModuleName.'/';
        $sAddonRel    = ''.$sModuleName.'/'.$sAddonName.'/';
        $sPattern     = "/^(.*?\/)".$sModuleName."\/.*$/";
        $sAppPath     = \preg_replace ($sPattern, "$1", $sModulesPath, 1 );

        $sInstallStruct  = $sAddonPath.'install-struct.sql.php';
        $sInstallData    = $sAddonPath.'install-data.sql.php';

        $oReg     = WbAdaptor::getInstance();
        $oDb      = $oReg->getDatabase();
        $oTrans   = $oReg->getTranslate();
        $oRequest = $oReg->getRequester();
        $oApp     = $oReg->getApplication();
        $oTrans->enableAddon(ADMIN_DIRECTORY.'\\\addons');

        $msg = [];
        $sErrorMsg = null;
        $OK   = "<span class=\"ok\">OK</span>";
        $FAIL = "<span class=\"error\">FAILED</span>";

        $sCallingScript = $oRequest->getServerVar('SCRIPT_NAME');
        $globalStarted = \preg_match('/upgrade\-script\.php$/', $sCallingScript);
        $sTableName = $oReg->TablePrefix.'mod_wbbackup_settings';

        $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : $oReg->AppVersion);
        $sModulePlatform = PreCheck::getAddonVariable($sAddonName,'platform');

    try {

    // create tables from sql dump file
        if (is_readable($sInstallStruct)) {
            $oDb->SqlImport($sInstallStruct, TABLE_PREFIX, 'install' );
        }
        if ($oDb->is_error()) {
            //$admin->print_error($database->get_error());
            $sMsg = (sprintf("%s\n",$oDb->get_error()));
            throw new \Exception ($sMsg);
        }

        // import setting values tables from sql dump file
        if (is_readable($sInstallData)) {
            $oDb->SqlImport($sInstallData, TABLE_PREFIX, 'install' );
        }
        if ($oDb->is_error()) {
            //$admin->print_error($database->get_error());
            $sMsg = (sprintf("%s\n",$oDb->get_error()));
            throw new \Exception ($sMsg);
        }

        } catch (\Exception $e) {
            if (!$globalStarted){
                $sLink = '<a class="back-link" href="'.$oReg->AcpUrl.'modules/index.php?advanced=1'.'" >'.$oTrans->OVERVIEW_ADDONS.'</a>';
                $oApp->ShowMaintainScreen('error',$e->getMessage(),$sLink,false);
            } else {
                echo nl2br(sprintf($e->getMessage()));
            }
            exit;
        }
    }
}
// ------------------------------------

// this var comes from outside
    $bDebugModus = ((isset($bDebugModus)) ? $bDebugModus : false);
    if (is_array($msg = mod_wbBackup_install($bDebugModus))) {
        if (!$bDebugModus) {
            echo '<b>'.implode('<br />',$msg).'</b><br />';
        }
    }

// end of file

