<?php
/**
 *
/**
 * @category     Addons
 * @package      Admin tool: wbBackup
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.2
  * @lastmodified 2021/03/05
 * @since        File available since 2021/01/14
 * @description  display backend administration page
 * SaveSetting.inc.php
 */

use addon\wbBackup\Lib\Wbbackup;
use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck};

use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\TwigFilter;

/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
/* -------------------------------------------------------- */

//load some initial datas
    if (is_readable($sAddonPath.'/init.php')) {require ($sAddonPath.'/init.php');}
// if autoload failed
    // if (!class_exists('Wbbackup')){require $sAddonPath.'/Lib/Wbbackup.php';}
    $oTrans->enableAddon('modules\\\\'.$sAddonName);

    // back to overview
    $backlink = ADMIN_URL .'/admintools/tool.php?tool="wbBackup';
    //  FTAN-CHECK
    $sSettingsID = $admin->checkIDKEY('SettingsID', false, 'POST');
        // Get id
    if(!$sSettingsID) {
        $admin->print_error('SaveSettings.inc.php '.$MESSAGE['GENERIC_SECURITY_ACCESS'], $backlink);
        exit();
    }
    $newSettings = [];
    $aErrorMsg = [];
    //Daten aus $aRequestVars auswerten und validieren
//    $newSettings['section_id'] = $section_id;

    if(isset($aRequestVars['cmd']) && ($aRequestVars['cmd'] == 'savesetting')) {

 // backup folder (backup_dir) - default = 'Backups'
        if (isset($aRequestVars['backup_dir']) && !empty(trim($aRequestVars['backup_dir']))) {
            $newSettings['backup_dir']          = $admin->StripCodeFromText($admin->get_post('backup_dir'));
            $newSettings['backup_dir']          = stripslashes($newSettings['backup_dir']);
        }else{
            $newSettings['backup_dir']          = 'Backups';
        }

 // excluded folders (skipped in the file backup (excluded_dirs))
        $aDefaultBackupFolder = ['Backups'];
        $sExcludedDirs = (empty(trim($aRequestVars['excluded_dirs'])) ? $newSettings['backup_dir'] : $aRequestVars['excluded_dirs']);
        $sExcludedDirs = $admin->StripCodeFromText($sExcludedDirs);
        if (!\is_array($sExcludedDirs)){ // \/ pregsplit without slash
            $aExcludedDirs             = \preg_split('/[\s,=+\;\:\|]+/', $sExcludedDirs, -1, PREG_SPLIT_NO_EMPTY);
        }
        $aExcludedDirsValue            = \array_merge($aDefaultBackupFolder, $aExcludedDirs);
        $aExcludedDirsValue            = \array_unique($aExcludedDirsValue);
        $newSettings['excluded_dirs']  = \trim(\implode(',', $aExcludedDirsValue),',');

// excluded_extension (skipped in the file backup (excluded_extension))
        $aDefaultFileExt = [''];
        $sExcludedExt = $admin->StripCodeFromText($aRequestVars['excluded_extension']);
        if (!\is_array($sExcludedExt)){
            $sExcludedExt                      = \preg_split('/[\s,=+\;\:\/\|]+/', $sExcludedExt, -1, PREG_SPLIT_NO_EMPTY);
        }
        $sExcludedExtValue                     = \array_merge($aDefaultFileExt, $sExcludedExt);
        $sExcludedExtValue                     = \array_unique($sExcludedExtValue);
        $newSettings['excluded_extension']     = \trim(\implode(',', $sExcludedExtValue),',');

// use_log (write a LOG about the file-backup, yes or no) - default=1(yes)
        if (isset($aRequestVars['use_dot']) && ($admin->StripCodeFromText($admin->get_post('use_dot'))))  {
            $newSettings['use_dot'] = $aRequestVars['use_dot'];
        } else {
            $newSettings['use_dot'] = 'true';
        }

// use_log (write a LOG about the file-backup, yes or no) - default=1(yes)
        if (isset($aRequestVars['use_log']) && ($admin->StripCodeFromText($admin->get_post('use_log'))))  {
            $newSettings['use_log'] = $aRequestVars['use_log'];
        } else {
            $newSettings['use_log'] = 'true';
        }
// log_excluded_dirs (write in the LOG the name of excluded dirs) - default=1(yes)
        if (isset($aRequestVars['log_excluded_dirs']) && ($admin->StripCodeFromText($admin->get_post('log_excluded_dirs'))))  {
            $newSettings['log_excluded_dirs'] = $aRequestVars['log_excluded_dirs'];
        } else {
            $newSettings['log_excluded_dirs']   = 'true';
        }
// log_excluded_large_files (rite in the LOG the name of excluded large files) - default=1(yes)
        if (isset($aRequestVars['log_excluded_large_files']) && ($admin->StripCodeFromText($admin->get_post('log_excluded_large_files'))))  {
            $newSettings['log_excluded_large_files'] = $aRequestVars['log_excluded_large_files'];
        } else {
            $newSettings['log_excluded_large_files'] = 'true';
        }
// max_file_size - (files with this size are excluded from the backup) - default=20 gb
        if (isset($aRequestVars['max_file_size']) && is_numeric($aRequestVars['max_file_size']) ) {
            $newSettings['max_file_size']= $aRequestVars['max_file_size'];
        } else {
            $newSettings['max_file_size'] = 20; // Mb
        }

// use_droptable / ( use DROP TABLE in mysql-backup, needed to overwrite an existed database - yes or no) - default=1
        if (isset($aRequestVars['use_droptable']) && ($admin->StripCodeFromText($admin->get_post('use_droptable'))))  {
            $newSettings['use_droptable'] = $aRequestVars['use_droptable'];
        } else {
            $newSettings['use_droptable'] = 'true';
        }
// use_compress / ( use DROP TABLE in mysql-backup, needed to overwrite an existed database - yes or no) - default=1
        if (isset($aRequestVars['use_compress']) && ($admin->StripCodeFromText($admin->get_post('use_compress'))))  {
            $newSettings['use_compress'] = $aRequestVars['use_compress'];
        } else {
            $newSettings['use_droptable'] = 'None';
        }
// use_character_set / ( use DROP TABLE in mysql-backup, needed to overwrite an existed database - yes or no) - default=1
        if (isset($aRequestVars['use_character_set']) && ($admin->StripCodeFromText($admin->get_post('use_character_set'))))  {
            $newSettings['use_character_set'] = $aRequestVars['use_character_set'];
        } else {
            $newSettings['use_character_set'] = 'utf8';
        }

//SQL wich is used for all updates
        $rawUpdtSQL = "
        UPDATE `".$oReg->TablePrefix."mod_wbbackup_settings`
         SET `s_value` = '%s' WHERE `s_name` = '%s';
         ";
        // execute SQL
        $sUpdateSql =[];
//die('Stop Save Settings');
        foreach ($newSettings as $s_name=>$s_value){
            if (!is_array($s_value)) {
                $sSql = sprintf($rawUpdtSQL, $database->escapeString($s_value), $s_name);
            } else {
                $sSql = sprintf($rawUpdtSQL, serialize($s_value), $s_name);
            }
            $sUpdateSql[] = $sSql;

            if (!$database->query($sSql)){
                $aErrorMsg[] = $database->get_error();
            }
        }

        if (sizeof($aErrorMsg)){
          $admin->print_error(implode('<br />', $aErrorMsg), $backlink);
        }
        $FTAN = $admin->getFTAN('');
        // complete array with replacements
        $redirect_timer = ((\defined( 'REDIRECT_TIMER' ) && (REDIRECT_TIMER <= 10000)) ? REDIRECT_TIMER : false);
        $aTplDefaults = array(
                'WB_URL'                        => WB_URL,
                'ADMIN_URL'                     => ADMIN_URL,
                'FTAN_NAME'                     => $FTAN['name'],
                'FTAN_VALUE'                    => $FTAN['value'],
                'MODULE_THEME_URL'              => $sAddonThemeUrl,
                'MODULE_THEME_PATH'             => $sAddonThemePath,
                'ICON_URL'                      => $sAddonThemeUrl.'/images',
                'ADDON_NAME'                    => strtolower($sAddonName),
                'ADDONNAME'                     => strtoupper($sAddonName),
                'ADDONURL'                      => $sAddonUrl,
                'LANGUAGE'                      => LANGUAGE,
                'ICON_URL'                      => $sAddonThemeUrl.'/images',
                'THEME_URL'                     => $sAddonThemeUrl,
                'BACKUP_URL'                    => $sAddonUrl.'/cmd/Backup.inc.php',
                'DELETE_BACKUP_URL'             => $sAddonUrl.'/cmd/Deletebackup.inc.php?file=',
                'SAVE_URL'                      => ADMIN_URL.'/admintools/tool.php?tool=wbBackup',
                'TOOL_URL'                      => ADMIN_URL.'/admintools/tool.php?tool=wbBackup',
                'REDIRECT_TIMER'                => $redirect_timer,
                'REDIRECT_URL'                  => ADMIN_URL.'/admintools/tool.php?tool=wbBackup',
                'SUCCESSFULL_SAVED'             => $oTrans->MOD_BACKUP_SUCCESSFULL_SAVED ,
                'TEXT_CLOSE'                    => $oTrans->MOD_BACKUP_TEXT_CLOSE ,
            );

    $aTwigdata = $aTplDefaults;

//    $get_backup_files('backups');
    if (version_compare($sWbVersion, '2.13.0', '>=')){
        if (is_readable($sAddonThemePath.'/success.twig'))
            {
                // create twig template object
                $loader = new FilesystemLoader($sAddonThemePath);
                $oTwig   = new Environment($loader, [
                    'autoescape'       => false,
                    'cache'            => false,
                    'strict_variables' => false,
                    'debug'            => false,
                    'auto_reload'      => true,
                ]);
            }
//            $oTwig->addGlobal('lang', array_merge($TEXT, $MOD_BACKUP));
            //$oTwig->addFilter(new TwigFilter('strftime', 'strftime_filter'));
            $oTwig->addFilter(new TwigFilter('var_dump','var_dump'));
            echo $oTwig->render('success.twig', $aTwigdata);
        }else{
            $oTwigLoader = new Twig_Loader_Filesystem($sAddonThemePath);
            $oTwig = new Twig_Environment($oTwigLoader, array(
                'autoescape'       => false,
                'cache'            => false,
                'strict_variables' => false,
                'debug'            => true
            ));
            $oTwig->addGlobal('lang', array_merge($TEXT, $MOD_BACKUP));
            $oTpl = $oTwig->loadTemplate('success.twig');
            $oTpl->display($aTwigdata);
        }

    }
//end of file

