<?php


use bin\{WbAdaptor,SecureTokens,Sanitize,WbDatabase};
use bin\helpers\{PreCheck,msgQueue};
use src\Security\{CsfrTokens,Randomizer};
use src\Interfaces\Requester;
use bin\Requester\HttpRequester;


/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
/* -------------------------------------------------------- */

if (!\is_callable('mod_wbBackup_upgrade')){
    function mod_wbBackup_upgrade($bDebug=false) {
        //global $OK ,$FAIL; // from core upgrade-script
        $sAddonFile   = \str_replace('\\','/',__FILE__);
        $sFolder      = \basename(\dirname($sAddonFile));
        switch ($sFolder):
            case 'cmd':
              $sAddonPath   = \dirname($sAddonFile,2).'/';
              break;
            default :
              $sAddonPath   = (\dirname($sAddonFile)).'/';
        endswitch;

        $sModulesPath = \dirname($sAddonPath).'/';
        $sModuleName  = \basename($sModulesPath);
        $sAddonName   = \basename($sAddonPath);
        $ModuleRel    = ''.$sModuleName.'/';
        $sAddonRel    = ''.$sModuleName.'/'.$sAddonName.'/';
        $sPattern     = "/^(.*?\/)".$sModuleName."\/.*$/";
        $sAppPath     = \preg_replace ($sPattern, "$1", $sModulesPath, 1 );

        $oReg     = WbAdaptor::getInstance();
        $oDb      = $oReg->getDatabase();
        $oTrans   = $oReg->getTranslate();
        $oRequest = $oReg->getRequester();
        $oApp     = $oReg->getApplication();
        $oTrans->enableAddon(ADMIN_DIRECTORY.'\\\addons');

        $msg = [];
        $sErrorMsg = null;
        $OK   = "<span class=\"ok\">OK</span>";
        $FAIL = "<span class=\"error\">FAILED</span>";

        $sCallingScript = $oRequest->getServerVar('SCRIPT_NAME');
        $globalStarted = \preg_match('/upgrade\-script\.php$/', $sCallingScript);
        $sTableName = $oReg->TablePrefix.'mod_wbbackup_settings';
        $sInstallStruct  = $sAddonPath.'install-struct.sql.php';

        $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : $oReg->AppVersion);
        $sModulePlatform = PreCheck::getAddonVariable($sAddonName,'platform');

        try {
            $aNamespaces = \bin\CoreAutoloader::getNamespaces();
            //if (!in_array('Ifsnop/Mysqldump', $aNamespaces)){
            if (!is_dir($oReg->AppPath.'include/ifsnop')){
                $msg[] = $sMsg = (sprintf("%s <a href='%s'>Small_Fixes_for_WB2_13_x</a>\n",'Class Mysqldump is missing. Please download and install ','https://addon.websitebaker.org/pages/en/browse-add-ons.php?download=0099E945'));
                throw new \Exception ($sMsg);
            }

        if (\version_compare($sWbVersion, $sModulePlatform, '<')){
            $msg[] = $sErrorMsg = \sprintf('It is not possible to upgrade from WebsiteBaker Versions before %s',$sModulePlatform);
            if ($globalStarted){
                echo $sErrorMsg;
            } else {
                throw new \Exception ($sErrorMsg);
            }
            } else {
                $aDefaultSettings = [
                    'max_file_size' => '512',
                    'use_log' => 'true',
                    'use_dot' => 'true',
                    'excluded_dirs' => 'Backups,backups',
                    'use_droptable' => 'true',
                    'backup_dir' => 'Backups',
                    'log_excluded_dirs' => 'true',
                    'log_excluded_large_files' => 'true',
                    'excluded_extension' => 'zip',
                    'use_compress' => 'Gzip',
                    'use_character_set' => 'utf8'
                ];

                if (!$oDb->table_exists($sTableName)) {
                    $oDb->SqlImport($sInstallStruct, TABLE_PREFIX, 'upgrade' );
                }
                $sql = 'SELECT * FROM `'.$sTableName.'`';
                if (($oRes = $oDb->query($sql))){
                    $aTmpSettings = [];
                    $aOldSettings = ($oRes->fetchAll() ?? []);
                    foreach ($aOldSettings as $index => $aItem){
                         if ($aItem['section_id']){continue;}
                         $aTmpSettings[$aItem['s_name']] = $aItem['s_value'];
                    }
                    $aNewSetting = array_replace_recursive($aDefaultSettings,$aTmpSettings);
                }

                $sInstallStruct = $sAddonPath.'install-struct.sql.php';
                if ( !\is_readable($sInstallStruct)) {
                    $msg[] = '<b>\'missing or not readable file [install-struct.sql]\'</b> '.$FAIL;
                    $iErr = true;
                } else {

                    if (sizeof($aNewSetting)!= sizeof($aTmpSettings)){
                        $sNameValPairs = '';
                        foreach ($aNewSetting as $index => $val) {
                            $sNameValPairs .= ', (\''.$index.'\', \''.$oDb->escapeString($val).'\')';
                        }
                        $sValues = \ltrim($sNameValPairs, ', ');
                        $sql = '
                        REPLACE INTO `'.$sTableName.'` (`s_name`, `s_value`)
                        VALUES '.$sValues.'
                        ';
                        if (!$oDb->query($sql)) {
                            $msg[] = '<strong>\'WbBackup update new settings\'</strong> '.$FAIL;
                            $iErr = true;
                        }
                    }
                }
                $aFilesToDelete = [
                        '/Lib/class.Wbbackup.php',
                        '/Lib/db_backup.php',
                        '/Lib/ftp-backup.php',
                        '/cmd/Toolsavesetting.inc.php',
                        '/cmd/Upgrade.inc.php',
                    ];
                PreCheck::deleteFiles($sAddonPath,$aFilesToDelete);
            }
        } catch (\Exception $e) {
            if (!$globalStarted){
                $sLink = '<a class="back-link" href="'.$oReg->AcpUrl.'modules/index.php?advanced=1'.'" >'.$oTrans->OVERVIEW_ADDONS.'</a>';
                $oApp->ShowMaintainScreen('error',$e->getMessage(),$sLink,false);
            } else {
                echo nl2br(sprintf($e->getMessage()));
            }
            exit;
        }

    }
}
// ------------------------------------

// this var comes from outside
    $bDebugModus = ((isset($bDebugModus)) ? $bDebugModus : false);
    if (is_array($msg = mod_wbBackup_upgrade($bDebugModus))) {
        if (!$bDebugModus) {
            echo '<b>'.implode('<br />',$msg).'</b><br />';
        }
    }
