<?php
/**
 * @category     Addons
 * @package      Admin tool: wbBackup
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com> (last), Dietmar Wöllbrink <dietmar.woellbrink@websitebaker@org>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.2
 * @lastmodified 2021/03/05
 * @since        File available since 2021/01/14
 * @description  display backend administration page
 *
*/

use addon\wbBackup\Lib\Wbbackup;
use bin\{SecureTokens,WbAdaptor};
use bin\helpers\{PreCheck,ParenList};


// only included in init.php may be needed as global if script is called within a function
global  $sAppPath,
//        $sAddonPath,
        $sAddonUrl,
        $sAddonName,
        $sAddonThemeUrl,
        $sAddonTemplateUrl,
        $sAddonThemePath,
        $sAddonRel,
        $aExtraFields,
        $oReg;
/* -------------------------------------------------------- */
// folderstructure steps to modules path
//$sAppPath = dirname(dirname(__DIR__));
// load config created for modify or save module files if WB_PATH don't exist, '
//if ( !defined( 'WB_PATH' ) ){require($sAppPath.'/config.php');}
// Must include code to prevent this file from being accessed directly
if (defined('WB_PATH') == false) { die('Cannot access '.basename(__DIR__).'/'.basename(__FILE__).' directly'); }
/* -------------------------------------------------------- */

$sCallingScript = $oReg->Request->getServerVar('SCRIPT_NAME');
$globalStarted = \preg_match('/upgrade\-script\.php$/', $sCallingScript);
$sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : WB_VERSION);

    // needed for simple Dispatcher
    $sAddonName = basename($sAddonPath);
    $bIsBackend = ($oApp instanceof admin);

    // An associative array that by default contains the contents of $_GET, $_POST and $_COOKIE.
    $aRequestVars = $_REQUEST;

    if (!function_exists('index_exists'))
    {
        function index_exists($table_name, $index_name, $number_fields = 0)
        {
            $bRetval = false;
            $oReg = WbAdaptor::getInstance();
            $oDb  = $oReg->getDatabase();
            $number_fields = intval($number_fields);
            $keys = 0;
            $sql = 'SHOW INDEX FROM `'.$table_name.'`';
            if (($res_keys = $oDb->query($sql))) {
                while (($rec_key = $res_keys->fetchRow(MYSQLI_ASSOC))) {
                    if ( ($rec_key['Key_name'] || ($rec_key['Column_name']) == $index_name) ) {
                        $keys++;
                    }
                }
            }
            if ( $number_fields == 0 ) {
                $bRetval =  ($keys != $number_fields);
            } else {
                $bRetval =  ($keys == $number_fields);
            }
            return $bRetval;
        }
    }

/* this function/closure is placed inside this file temporarely until a better place is found */
/*  function to update a var/value-pair(s) in table ****************************
 *  nonexisting keys are inserted
 *  @param string $table: name of table to use (without prefix)
 *  @param mixedthe firring with name of the key to update
 *  @param string $value: a sting with needed value, if $key is a string too
 *  @param array example the first entry is used as WHEE....... AND request
 *                  $aFieldsList = array(
 *                      'section_id' => $section_id,
 *                      'page_id' => $page_id,
 *                  );
 *
 *  @return bool:  true if any keys are updated, otherwise false
 */
    if (!function_exists('UpdateKeyValue'))
    {
        function UpdateKeyValue($table, $key, $value='', array $aExtraFields=[])
        {
            //global $database;
            if( !is_array($key))
            {
                if (\trim($key) != '' )
                {
                    $key = array( \trim($key) => \trim($value) );
                } else {
                    $key = [];
                }
            }
            $retval = [];
            $iFirst = 0;
            $aIndexField  = [];
            $sExtraFields = '';
            $aTmp = [];
            $oReg = WbAdaptor::getInstance();
            $oDb  = $oReg->getDatabase();
            $sql = 'SHOW FIELDS FROM `'.$oReg->TablePrefix.$table.'` ';
            if ($oRes = $oDb->query($sql)){
                while($aRes = $oRes->fetchRow(MYSQLI_ASSOC)){
                  if (!isset($aExtraFields[$aRes['Field']])){
                      continue;
                  }
                  $aTmp[$aRes['Field']] = $aExtraFields[$aRes['Field']];
                  if (index_exists($oReg->TablePrefix.$table, $aRes['Field'])){
                      $aIndexField[] = $aRes['Field'].' = '.$aTmp[$aRes['Field']];
                      $iFirst++;
                  }
                }
                $aExtraFieldsList =  \array_intersect_key($aTmp, $aExtraFields);
                foreach($aExtraFieldsList as $sName=>$sValue){
                    $sExtraFields .= '`'.$sName.'` = \''.$sValue.'\', ';
                }
            } else {
                $retval[]=$sql;
                $retval[]=$oDb->get_error();
            }
            foreach( $key as $index=>$val)
            {
    //            $index = strtolower($index);
                $sql  = 'SELECT COUNT(*) FROM `'.$oReg->TablePrefix.$table.'` '
                      . 'WHERE `s_name` = \''.$index.'\' '
                      . (($aIndexField['0']!='') ? 'AND '. $aIndexField['0'] : '');


                if ($oDb->get_one($sql))
                {
                    $sql = 'UPDATE ';
                    $sql_where = 'WHERE `s_name` LIKE \''.$index.'\'';
                } else {
                    $sql = 'INSERT INTO ';
                    $sql_where = '';
                }
                $sql .= '`'.$oReg->TablePrefix.$table.'` SET ';
                $sql .= (($sExtraFields) ? $sExtraFields : '');
                $sql .= '`s_name` = \''.$index.'\', ';
                $sql .= '`s_value` = \''.$val.'\' '.$sql_where;
                if (!$oDb->query($sql))
                {
                    $retval[]=$sql;
                    $retval[]=$oDb->get_error();
                }
            }
            return ((\sizeof($retval)==0) ? true : $retval);
        }
    }
#################################################################################

    $sModule = '
<script type="text/javascript">
var '.strtoupper($sAddonName).' = {
        ADMIN_DIRECTORY : "'.ADMIN_DIRECTORY.'",
        WB_URL : "'.$oReg->AppUrl.'",
        ADMIN_URL : "'.$oReg->AcpUrl.'",
        AddonUrl : "'.$sAddonUrl.'/",
        MODULE_THEME_URL : "'.$sAddonThemeUrl.'"
    };
    var ModulesCss = "'.$sAddonThemeUrl.'/css/default.css";
    if (typeof LazyOnLoad === \'undefined\'){
        $.insert(ModulesCss);
    } else {
        LazyOnLoad (\'head\', ModulesCss);
    }
</script>
';

    $aDefaults = [
        'section_id'                            => '0',
        'max_file_size'                         => '20',
        'use_log'                               => 'true',
        'excluded_dirs'                         => 'backups',
        'excluded_extension'                    => '',
        'use_droptable'                         => 'true',
        'backup_dir'                            => 'backups',
        'log_excluded_dirs'                     => 'true',
        'log_excluded_large_files'              => 'true',
    ];

// set any defaults
    $sAddonName = basename($sAddonPath);
    $sAddonPath = $oReg->AppPath.'modules/'.$sAddonName;
    $sAddonUrl  = $oReg->AppUrl.'modules/'.$sAddonName;
    $sAddonRel = '/modules/'.$sAddonName;
    $aInstance = [
          'ADDON_NAME' => strtoupper($sAddonName),
          'ADMIN_DIRECTORY' => ADMIN_DIRECTORY,
          'AppUrl' =>  $oReg->AppUrl,
          'AcpUrl' =>  $oReg->AcpUrl,
          'sAddonUrl' => $sAddonUrl,
          'sAddonThemeUrl'  => $sAddonUrl.'/themes/default',
          'ModulesTemplateUrl' => ($bIsBackend ? $sAddonThemeUrl : $sAddonTemplateUrl),
          'ADDON_URL'=> $sAddonUrl,
          'ModuleScript' => $sModule
          ];

// end of file

